package captcha

import (
	"fmt"
	"math/rand"
	"strings"
)

type Question struct {
	Prompt string
	Answer string
}

type Captcha []Question

func (captcha Captcha) Get() (int, string) {
	n := rand.Intn(len(captcha))
	return n, captcha[n].Prompt
}

func (captcha Captcha) Check(n int, ans string) bool {
	if n < 0 || n >= len(captcha) {
		return false
	}
	return ans == captcha[n].Answer
}

type Datum struct {
	Format  string
	Choices []string
	Answers []Clue
}

type Clue struct {
	Hint    string
	Correct int
}

/*
Tries to generate captchas from datasets like this in json.

	[ {
	"Format": "Pick the one which %s.",
	"Choices": [ "apple", "dog", "cat", "ghost" ],
	"Answers": [
		{ "Hint": "is a fruit", "Correct": 0 },
		{ "Hint": "barks", "Correct": 1 },
		{ "Hint": "is not alive", "Correct": 3 }
	]
	} ]
*/
func Generate(dataset []Datum) Captcha {
	var captcha Captcha
	for _, data := range dataset {
		for _, clue := range data.Answers {
			captcha = append(captcha, Question{
				Prompt: fmt.Sprintf(data.Format, clue.Hint) +
					" (" + strings.Join(data.Choices, ", ") + ")",
				Answer: data.Choices[clue.Correct],
			})
		}
	}
	return captcha
}
